{ V1.0
  TElegantThread For D7
  by 2013(QQ:26562729) 20180501
}
unit uElegantThread;
interface
uses
  uSimpleThread, uSimpleList, SyncObjs;
type

  PEpitRec = ^TEpitRec; // exec proc in Thread
  TEpitRec = record
    ProcKind: TProcKind;
    ObjectProc: TObjectProc;
    SimpleProc: TSimpleProc;
    Param: Pointer;
    Event: TEvent;
  end;

  TEpitList = class(TSimpleList)
  protected
    procedure FreeItem(Item: Pointer); override;
  end;

  // ͬ TSimpleThread ,TSimpleList
  //  proc һִ߳

  TElegantThread = class(TSimpleThread)
  private
    FEpitList: TEpitList;
    procedure Work;
    procedure DoWork;
  public
    constructor Create(AllowedActiveX: boolean = false); reintroduce;
    destructor Destroy; override;

    // Synchronize , AProc 뱾ִ߳У AProc ִнΪֹ
    procedure Synchronize(AProc: TObjectProc); overload;

    // ضԴ AParam ͷ
    procedure Synchronize(AProc: TSimpleProc; AParam: Pointer); overload;

    // Queue , AProc 뱾ִ߳Уֱӷ
    procedure Queue(AProc: TObjectProc); overload;

    // ضԴ AParam ͷ
    procedure Queue(AProc: TSimpleProc; AParam: Pointer); overload;

  end;

implementation

{ TEpitList }

procedure TEpitList.FreeItem(Item: Pointer);
begin
  inherited;
  Dispose(PEpitRec(Item));
end;

{ TElegantThread }

constructor TElegantThread.Create(AllowedActiveX: boolean);
begin
  inherited Create;
  FEpitList := TEpitList.Create;
end;

destructor TElegantThread.Destroy;
begin
  FEpitList.free;
  inherited;
end;

procedure TElegantThread.Queue(AProc: TObjectProc);
var
  P: PEpitRec;
begin
  FEpitList.Lock;

  New(P);
  FEpitList.Add(P);
  P.ProcKind := pkObjectProc;
  P.ObjectProc := AProc;
  P.Event := nil;

  FEpitList.Unlock;

  Work;

end;

procedure TElegantThread.Synchronize(AProc: TObjectProc);
var
  P: PEpitRec;
begin

  FEpitList.Lock;

  New(P);
  FEpitList.Add(P);
  P.ProcKind := pkObjectProc;
  P.ObjectProc := AProc;
  P.Event := TEvent.Create(nil, true, false, '');

  FEpitList.Unlock;

  Work;

  P.Event.WaitFor($FFFFFFFF);
  P.Event.free;

end;

procedure TElegantThread.DoWork;
var
  P: PEpitRec;

begin

  FEpitList.Lock;
  try
    P := FEpitList.PopFirst;
  finally
    FEpitList.Unlock;
  end;

  if Assigned(P) then
  begin
    try

      if P.ProcKind = pkObjectProc then
      begin
        P.ObjectProc;
      end
      else if P.ProcKind = pkSimpleProc then
      begin
        P.SimpleProc(P.Param);
      end;

    finally

      if Assigned(P.Event) then
      begin
        P.Event.SetEvent;
      end;

      Dispose(P);
      Work;
    end;

  end;

end;

procedure TElegantThread.Work;
begin
  ExeProcInThread(DoWork);
end;

procedure TElegantThread.Queue(AProc: TSimpleProc; AParam: Pointer);
var
  P: PEpitRec;
begin
  FEpitList.Lock;

  New(P);
  FEpitList.Add(P);
  P.ProcKind := pkSimpleProc;
  P.SimpleProc := AProc;
  P.Param := AParam;
  P.Event := nil;

  FEpitList.Unlock;

  Work;

end;

procedure TElegantThread.Synchronize(AProc: TSimpleProc; AParam: Pointer);
var
  P: PEpitRec;
begin

  FEpitList.Lock;

  New(P);
  FEpitList.Add(P);
  P.ProcKind := pkSimpleProc;
  P.SimpleProc := AProc;
  P.Param := AParam;
  P.Event := TEvent.Create(nil, true, false, '');

  FEpitList.Unlock;

  Work;

  P.Event.WaitFor($FFFFFFFF);
  P.Event.free;

end;

end.

