unit uSimpleList;

interface
uses
  Classes, SyncObjs;
type
  TSimpleList = class(TList)
  private
    FCS: TCriticalSection;
  protected
    procedure FreeItem(Item: Pointer); virtual; abstract;
  public
    procedure Lock;
    procedure Unlock;
    function PopFirst: Pointer;
    function PopLast: Pointer;
    procedure ClearAndFreeAllItems;
    constructor Create;
    destructor Destroy; override;
  end;

implementation

{ TSimpleList }

procedure TSimpleList.ClearAndFreeAllItems;
var
  i: integer;
begin
  for i := 0 to Self.Count - 1 do
    FreeItem(Self[i]);
  Clear;
end;

constructor TSimpleList.Create;
begin
  inherited;
  FCS := TCriticalSection.Create;
end;

destructor TSimpleList.Destroy;
begin
  Lock;
  try
    ClearAndFreeAllItems;
  finally
    Unlock;
  end;
  inherited;
end;

procedure TSimpleList.Lock;
begin
  FCS.Enter;
end;

function TSimpleList.PopFirst: Pointer;
begin
  Result := nil;
  if Self.Count > 0 then
  begin
    Result := Self[0];
    Self.Delete(0);
  end;
end;

function TSimpleList.PopLast: Pointer;
var
  n: integer;
begin
  Result := nil;
  n := Self.Count;
  if n > 0 then
  begin
    Result := Self[n - 1];
    Self.Delete(n - 1);
  end;
end;

procedure TSimpleList.Unlock;
begin
  FCS.Leave;
end;

end.

