{ V1.0
  TSimpleThread For D7
  by 2013(QQ:26562729) 20180501
}
unit uSimpleThread;
interface
uses
  Classes, SysUtils, SyncObjs;
type

  TObjectProc = TThreadMethod;
  TSimpleProc = procedure(Param: Pointer);

  TProcKind = (pkObjectProc, pkSimpleProc);

  TOnStatusMsg = procedure(AMsg: string) of object;
  TOnDebugMsg = TOnStatusMsg;
  // һ TSimple ϵĻ
  // Ҳ߳ӦʵĻ
  TSimpleThread = class(TThread)
  private

    FEvent: TEvent;
    FAllowedActiveX: boolean;
    FObjectProc: TObjectProc;
    FSimpleProc: TSimpleProc;
    FProcParam: Pointer;
    FProcKind: TProcKind;

    procedure DoExecute;
    procedure SelfStart;
    procedure DoOnThreadProcErr(E: Exception);

  protected
    FWaitStop: boolean;

    procedure BeforeExecute; virtual;
    procedure AfterExecute; virtual;
    procedure Execute; override;
    procedure SleepExceptStopped(ATimeOut: Cardinal);
    procedure WaitThreadStop;
    procedure OnThreadProcErr(E: Exception); virtual;

    procedure DoOnStatusMsg(AMsg: string);
    procedure DoOnDebugMsg(AMsg: string);

  public

    procedure BeforeDestruction; override;
    procedure StartThread; virtual;
    procedure StopThread;

    procedure ExeProcInThread(AProc: TObjectProc); overload;
    procedure ExeProcInThread(AProc: TSimpleProc; AParam: Pointer); overload;

    constructor Create(AllowedActiveX: boolean = false); reintroduce;
    destructor Destroy; override;

  public

    OnStatusMsg: TOnStatusMsg;
    OnDebugMsg: TOnDebugMsg;
    TagId: integer;

    property WaitStop: boolean read FWaitStop;

  end;

implementation
{ TSimpleThread }
uses
  Windows, ActiveX;

procedure TSimpleThread.AfterExecute;
begin
end;

procedure TSimpleThread.BeforeDestruction;
begin
  inherited;
  WaitThreadStop;
end;

procedure TSimpleThread.BeforeExecute;
begin
end;

constructor TSimpleThread.Create(AllowedActiveX: boolean);
begin
  inherited Create(false);
  FAllowedActiveX := AllowedActiveX;
  FreeOnTerminate := false;
  FEvent := TEvent.Create(nil, true, false, '');
end;

destructor TSimpleThread.Destroy;
begin
  FEvent.free;
  inherited;
end;

procedure TSimpleThread.DoExecute;
begin
  BeforeExecute;
  repeat
    FEvent.WaitFor(INFINITE);
    FEvent.ResetEvent;

    if not Terminated then
    begin
      try
        case FProcKind of
          pkObjectProc: FObjectProc;
          pkSimpleProc: FSimpleProc(FProcParam);
        end;
      except
        on E: Exception do
        begin
          DoOnThreadProcErr(E);
        end;
      end;
    end;

  until Terminated;
  AfterExecute;
end;

procedure TSimpleThread.DoOnDebugMsg(AMsg: string);
begin
  if Assigned(OnDebugMsg) then OnDebugMsg(AMsg);
end;

procedure TSimpleThread.DoOnStatusMsg(AMsg: string);
begin
  if Assigned(OnStatusMsg) then OnStatusMsg(AMsg);
end;

procedure TSimpleThread.DoOnThreadProcErr(E: Exception);
var
  sErrMsg: string;
begin
  sErrMsg := 'ClassName:' + ClassName + #13#10;
  sErrMsg := sErrMsg + 'TagID:' + IntToStr(TagId) + #13#10;
  sErrMsg := sErrMsg + 'ErrMsg:' + E.Message + #13#10;
  DoOnDebugMsg(sErrMsg);
  OnThreadProcErr(E);
end;

procedure TSimpleThread.Execute;
begin
  inherited;

  if FAllowedActiveX then
  begin
    CoInitialize(nil);
    try
      DoExecute;
    finally
      CoUninitialize;
    end;
  end
  else
    DoExecute;

end;

procedure TSimpleThread.ExeProcInThread(AProc: TObjectProc);
begin
  FObjectProc := AProc;
  FProcKind := pkObjectProc;
  SelfStart;
end;

procedure TSimpleThread.ExeProcInThread(AProc: TSimpleProc; AParam: Pointer);
begin
  FSimpleProc := AProc;
  FProcKind := pkSimpleProc;
  FProcParam := AParam;
  SelfStart;
end;

procedure TSimpleThread.OnThreadProcErr(E: Exception);
begin

end;

procedure TSimpleThread.SelfStart;
begin
  if FEvent.WaitFor(0) <> wrSignaled then
    FEvent.SetEvent; // waitfor ٵ
end;

procedure TSimpleThread.SleepExceptStopped(ATimeOut: Cardinal);
var
  BOldTime: Cardinal;
  LDiff: integer;
begin
  // sleep ʱ˳ָ,ȷ߳˳˳
  BOldTime := GetTickCount;
  while not WaitStop do
  begin
    LDiff := ATimeOut - (GetTickCount - BOldTime);
    if LDiff <= 0 then
      Break
    else if LDiff < 50 then
      Sleep(LDiff)
    else Sleep(50);
  end;
end;

procedure TSimpleThread.StartThread;
begin

end;

procedure TSimpleThread.StopThread;
begin
  FWaitStop := true;
end;

procedure TSimpleThread.WaitThreadStop;
begin
    // ȴ߳̽
  StopThread;
  Terminate;
  SelfStart;
  WaitFor;
end;

end.

